module Main where

import IO
import Maybe
import Formula
import FormulaParser
import HSLatexOutput
import System
import AtomicEval
import List

-- functions to decompose a hypersequent into a non-atomic formula and side
-- formulas resp. side r-hypersequents
extractFmlaL :: HSeq -> Maybe (HSeq, [Formula], Formula, ROp, [Formula])
extractFmlaL (HSeq rs)
	| (isNothing res) = Nothing
	| (isJust res) = Just (HSeq s, fsl, f, rop, fsr)
	where
	res = extractFromList (isJust.extRSeqL) rs
	(r,s) = fromJust res
	(fsl,f,rop,fsr) = fromJust $ extRSeqL r

extRSeqL :: RSeq -> Maybe ([Formula],Formula,ROp,[Formula])
extRSeqL (RSeq fsl rop fsr)
	| (isNothing res) = Nothing
	| (isJust res) = Just (sl,f,rop,fsr) 
	where
        res = extractFmla  fsl
	(f,sl) = fromJust res

extractFmlaR :: HSeq -> Maybe (HSeq, [Formula], ROp, Formula, [Formula])
extractFmlaR (HSeq rs)
	| (isNothing res) = Nothing
	| (isJust res) = Just (HSeq s, fsl, rop, f, fsr)
	where
	res = extractFromList (isJust.extRSeqR) rs
	(r,s) = fromJust res
	(fsl,rop,f,fsr) = fromJust $ extRSeqR r
	
extRSeqR :: RSeq -> Maybe ([Formula],ROp,Formula,[Formula])
extRSeqR (RSeq fsl rop fsr)
	| (isNothing res) = Nothing
	| (isJust res) = Just (fsl,rop,f,sr) 
	where
        res = extractFmla fsr
	(f,sr) = fromJust res

deriveFormula :: Formula -> Derivation
deriveFormula f = derive $ HSeq [RSeq [] Leq [f]]

derive :: HSeq -> Derivation
derive (HSeq ss)
	| (isJust resL) =  deriveL h $ fromJust resL
	| (isJust resR) =  deriveR h $ fromJust resR
	| otherwise = Leaf h
	where
	h = HSeq (nub ss)
	resL = extractFmlaL h
	resR = extractFmlaR h
	

-- the derivation rules
deriveR h (HSeq rs,fsl,rop,SAnd f1 f2,fsr) = 
	Der h ("sand",rop,'r') $ map derive [HSeq (rs ++ [RSeq fsl rop (Falsum:fsr), RSeq fsl rop (f1:f2:fsr)])]
	
deriveR h (HSeq rs,fsl,rop,Impl f1 f2,fsr) = 
	Der h ("impl",rop,'r') $ map derive [HSeq (rs ++ [RSeq fsl rop fsr]),
                                             HSeq (rs ++ [RSeq (fsl++[f1]) rop (f2:fsr), RSeq [f1] Leq [f2]])]

deriveR h (HSeq rs,fsl,rop,And f1 f2,fsr) = 
	Der h ("and",rop,'r') $ map derive [HSeq (rs ++ [RSeq fsl rop (f1:fsr)]),
	                                    HSeq (rs ++ [RSeq fsl rop (f2:fsr)])]

deriveR h (HSeq rs,fsl,rop,Or f1 f2,fsr) = 
	Der h ("or",rop,'r') $ map derive [HSeq (rs ++ [RSeq fsl rop (f1:fsr), RSeq fsl rop (f2:fsr)])]

deriveL h (HSeq rs,fsl,SAnd f1 f2,rop,fsr) = 
	Der h ("sand",rop,'l') $ map derive [HSeq (rs ++ [RSeq (fsl++[f1,f2]) rop fsr]),
	                                     HSeq (rs ++ [RSeq (fsl++[Falsum]) rop fsr])]

deriveL h (HSeq rs,fsl,Impl f1 f2,rop,fsr) = 
	Der h ("impl",rop,'l') $ map derive [HSeq (rs ++ [RSeq fsl rop fsr, RSeq (fsl++[f2]) rop (f1:fsr)]),
	                                     HSeq (rs ++ [RSeq fsl rop fsr, RSeq [f2] Lt [f1]])]

deriveL h (HSeq rs,fsl,And f1 f2,rop,fsr) = 
	Der h ("and",rop,'l') $ map derive [HSeq (rs ++ [RSeq (fsl++[f1]) rop fsr, RSeq (fsl++[f2]) rop fsr])]

deriveL h (HSeq rs,fsl,Or f1 f2,rop,fsr) = 
	Der h ("or",rop,'l') $ map derive [HSeq (rs ++ [RSeq (fsl ++ [f1]) rop fsr]),
	                                   HSeq (rs ++ [RSeq (fsl ++ [f1]) rop fsr])]

usage = "Usage: hypseq <formula>\n" ++
        "eg: hypseq \"a & (b -> c)\"\n" ++
	"    hypseq \"(d | a) /\\  d)\"\n" ++
	"    hypseq \"p -> (q -> (p & q))\"\n"


main = do
	s <- getArgs
--	s <- getLine
--	f <- parseFormula s
        if (null s || head s == "") then do
			hPutStr stderr usage 
      	                exitFailure 
	 else do f <- parseFormula $ head s
                 if (isNothing f) then exitFailure
                  else do putStrLn outputPrefix
                          let df = deriveFormula (fromJust f) in 
                            do putStrLn $ show $ df
                         --      if evalDerivation evalLuk df then 
                         --           putStrLn "The formula is valid for every truth assignment (in \\L)." 
                         --       else putStrLn "The formula is not valid for every truth assignment (in \\L)." 
                          putStrLn outputSuffix


